package server

import (
	"context"
	"fmt"
	"math/rand"
	"strconv"
	"strings"

	"github.com/gofiber/fiber/v2"
	"github.com/rs/zerolog/log"

	"pluja.dev/kycnot.me/database"
	"pluja.dev/kycnot.me/ent/service"
	"pluja.dev/kycnot.me/utils"
)

func (s *Server) handleIndex(c *fiber.Ctx) error {
	nokycPf := []string{
		"KYC-Free Crypto Freedom.",
		"Goodbye KYC, hello privacy.",
		"KYC is a scam. Don't fall for it.",
	}

	t := c.Query("t", "exchange")
	q := c.Query("q", "")

	// Currencies
	xmr := c.Query("xmr", "")
	btc := c.Query("btc", "")
	ln := c.Query("ln", "")
	cash := c.Query("cash", "")
	fiat := c.Query("fiat", "")

	// Query all services from the database and filter them by the query and the currencies
	queryBuilder := database.Client.Service.Query().Where(service.Listed(true), service.Pending(false))

	// Apply the type filter if present
	if t != "" && t != "all" {
		log.Printf("Type: %v", t)
		queryBuilder = queryBuilder.Where(service.TypeEQ(service.Type(t)))
	}

	// Apply the query text filter if present
	if q != "" {
		log.Printf("Query: %v", q)
		queryBuilder = queryBuilder.Where(
			service.Or(
				service.NameContains(q),
				service.DescriptionContains(q),
				service.TagsContains(q),
			),
		)
	}

	// Apply the currency filters if present
	if xmr != "" {
		queryBuilder = queryBuilder.Where(service.XmrEQ(xmr == "on"))
	}
	if btc != "" {
		queryBuilder = queryBuilder.Where(service.BtcEQ(btc == "on"))
	}
	if ln != "" {
		queryBuilder = queryBuilder.Where(service.LightningEQ(ln == "on"))
	}
	if cash != "" {
		queryBuilder = queryBuilder.Where(service.CashEQ(cash == "on"))
	}
	if fiat != "" {
		queryBuilder = queryBuilder.Where(service.FiatEQ(fiat == "on"))
	}

	// Execute the query
	services, err := queryBuilder.All(context.Background())
	if err != nil {
		// Handle error (e.g., log it and return a generic error message to the client)
		return c.Status(fiber.StatusInternalServerError).SendString("Internal Server Error")
	}

	return c.Render("index", fiber.Map{
		"Title": "Home",
		"Filters": map[string]string{
			"Type":  t,
			"Query": q,
			"Xmr":   xmr,
			"Btc":   btc,
			"Ln":    ln,
			"Cash":  cash,
			"Fiat":  fiat,
		},
		"Current":     "index",
		"Services":    services,
		"RandomPitch": nokycPf[rand.Intn(len(nokycPf))],
	}, "base")
}

func (s *Server) handleService(c *fiber.Ctx) error {
	serviceName := strings.ToLower(c.Params("name"))

	// Get service from database by name
	service, err := database.Client.Service.Query().Where(service.NameEQ(serviceName)).First(context.Background())
	if err != nil {
		log.Error().Err(err).Msgf("Could not get service %v from database", serviceName)
		return c.Status(fiber.StatusInternalServerError).SendString("Internal Server Error")
	}

	log.Printf("Service: %v", serviceName)
	return c.Render("service", fiber.Map{
		"Title":   fmt.Sprintf("%v | Service", serviceName),
		"Service": service,
	}, "base")
}

func (s *Server) handleRequestServiceForm(c *fiber.Ctx) error {
	challenge, id, difficulty, err := s.PowChallenger.PowGenerateChallenge(16)
	if err != nil {
		return err
	}
	return c.Render("request_service", fiber.Map{
		"Title":   "Request a service",
		"Current": "request",
		"Pow": fiber.Map{
			"Challenge":  challenge,
			"Difficulty": difficulty,
			"Id":         id,
		},
		"Error": c.Query("error", ""),
	}, "base")
}

func (s *Server) handleRequestServicePostForm(c *fiber.Ctx) error {
	nonce, id := c.FormValue("pow-nonce"), c.FormValue("pow-id")
	log.Printf("Nonce: %v, ID: %v", nonce, id)
	if !s.PowChallenger.PowVerifyProof(id, nonce) {
		return c.Redirect("/request/service?error=invalid-captcha")
	}

	// KYC Level
	log.Printf("KYC Level: %v", c.FormValue("kyc_level"))
	klInt, err := strconv.Atoi(c.FormValue("kyc_level"))
	log.Printf("KYC Level Int: %v", klInt)
	if err != nil || klInt < 0 || klInt >= 4 {
		log.Error().Err(err).Msgf("Invalid KYC Level value: %v", c.FormValue("kyc_level"))
		return c.Redirect("/request/service?error=invalid-kyc-level")
	}

	var serviceType service.Type
	switch c.FormValue("type") {
	case "exchange":
		serviceType = service.TypeExchange
	case "service":
		serviceType = service.TypeService
	case "wallet":
		serviceType = service.TypeWallet
	default:
		return c.Redirect("/request/service?error=invalid-service-type")
	}

	// Parse tags
	tags := strings.ReplaceAll(c.FormValue("tags"), " ", ",")
	// Remove trailing commas
	tags = strings.TrimSuffix(tags, ",")
	// Remove duplicate commas
	tags = strings.ReplaceAll(tags, ",,", ",")
	// Remove leading commas
	tags = strings.TrimPrefix(tags, ",")
	// Convert to lowercase
	tags = strings.ToLower(tags)

	_, err = database.Client.Service.Create().
		SetName(strings.ToLower(c.FormValue("name"))).
		SetDescription(c.FormValue("description")).
		SetType(serviceType).
		SetLogoURL(utils.UrlParser(c.FormValue("logo_url"))).
		SetUrls(utils.UrlListParser(c.FormValue("urls"))).
		SetTosUrls(utils.UrlListParser(c.FormValue("tos_urls"))).
		SetOnionUrls(utils.UrlListParser(c.FormValue("tor_urls"))).
		SetTags(tags).
		SetXmr(c.FormValue("xmr") == "on").
		SetBtc(c.FormValue("btc") == "on").
		SetLightning(c.FormValue("ln") == "on").
		SetFiat(c.FormValue("fiat") == "on").
		SetCash(c.FormValue("cash") == "on").
		SetKycLevel(klInt).
		SetPending(true).
		Save(c.Context())
	if err != nil {
		log.Error().Err(err).Msg("Could not save service to database")
		return c.Redirect("/request/service?error=internal-error")
	}

	return c.Redirect("/request/service?message=success")
}
